package com.termcat.util;

import java.io.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author
 * @version 1.0
 */

public class FileUtil
{

/**
* Allows cross-platform compatibility.
*/
  public static final String lineFeed = System.getProperty("line.separator");

  /**
  * Copy files and/or directories.
  *
  * @param src source file or directory
  * @param dest destination file or directory
  * @exception IOException if operation fails
  */
  public static final void copy(File src, File dest) throws IOException
  {
    FileInputStream source = null;
    FileOutputStream destination = null;
    byte[] buffer;
    int bytes_read;

    // Make sure the specified source exists and is readable.
    if (!src.exists())
    {
    throw new IOException("Source not found: " + src);
    }

    if (!src.canRead())
    {
    throw new IOException("Source is unreadable: " + src);
    }

    if (src.isFile())
    {
      if (!dest.exists())
      {
        File parentdir = parent(dest);
        if (!parentdir.exists())
        {
           parentdir.mkdir();
        }
      }
      else if (dest.isDirectory())
      {
        dest = new File(dest + File.separator + src);
      }
    }
    else if (src.isDirectory())
    {
      if (dest.isFile())
      {
         throw new IOException("Cannot copy directory " + src + " to file " + dest);
      }

      if (!dest.exists())
      {
        dest.mkdir();
      }
    }

    if (src.isFile())
    {
      try
      {
        source = new FileInputStream(src);
        destination = new FileOutputStream(dest);
        buffer = new byte[1024];
        while (true)
        {
          bytes_read = source.read(buffer);
          if (bytes_read == -1)
          {
            break;
          }
          destination.write(buffer, 0, bytes_read);
        }
      }
      finally
      {
        if (source != null)
        {
          try
          {
            source.close();
          }
          catch (IOException e) {}
        }
        if (destination != null)
        {
          try
          {
             destination.close();
          }
          catch (IOException e) {}
        }
      }
    }
    else if (src.isDirectory())
    {
      String targetfile, target, targetdest;
      String[] files = src.list();

      for (int i = 0; i < files.length; i++)
      {
        targetfile = files[i];
        target = src + File.separator + targetfile;
        targetdest = dest + File.separator + targetfile;

        if ((new File(target)).isDirectory())
        {
           copy(new File(target), new File(targetdest));
        }
        else
        {
          try
          {
            source = new FileInputStream(target);
            destination = new FileOutputStream(targetdest);
            buffer = new byte[1024];

            while (true)
            {
              bytes_read = source.read(buffer);
              if (bytes_read == -1)
              {
                break;
              }
              destination.write(buffer, 0, bytes_read);
            }
          }
          finally
          {
            if (source != null)
            {
              try
              {
                source.close();
              }
              catch (IOException e) {}
            }
            if (destination != null)
            {
              try
              {
                destination.close();
              }
              catch (IOException e) {}
            }
          }
        }
      }
    }
  }

  /**
  * Deletes a file or a directory along with all files and directories
  * it contains.
  *
  * @param file the file or directory to delete
  * @return whether delete was successful
  */
  public static final boolean deleteFile(File file)
  {
    if (file == null)
    {
      return false;
    }

    if (file.isDirectory())
    {
      File[] dirFileListing = file.listFiles();

      // For each file/directory in listing, make recursive call.
      int len = dirFileListing.length;
      for (int i = 0; i < len; i++)
      {
        if (!deleteFile(dirFileListing[i]))
        {
          // Break and return an error.
          return false;
        }
      }
    }

    // Delete file or directory.
    if (!file.delete())
    {
     // Display message and return an error.
      System.out.println("Could not delete: " + file.getAbsolutePath());
      return false;
    }

    return true;
  }

  /**
  * File.getParent() can return null when the file is specified without
  * a directory or is in the root directory. This method handles those
  * cases.
  *
  * @param file the target file to analyze
  * @return the parent directory as a file
  */
  private static final File parent(File file)
  {
    String dirname = file.getParent();
    if (dirname == null)
    {
      if (file.isAbsolute())
      {
        return new File(File.separator);
      }
      else
      {
        return new File(System.getProperty("user.dir"));
      }
    }
    return new File(dirname);
  }

  /**
  * Returns a string containing the contents of a file.
  *
  * @param file the file to read
  * @return the contents of the file
  */
  public static final String readFile(File file)  throws IOException
  {
    return readFile(file, null);
  }
  
  
  /**
   * Returns a string containing the contents of a file.
   *
   * @param file the file to read
   * @param encoding  the encoding to use, null means platform default
   * @return the contents of the file
   */
  public static final String readFile(File file, String encoding)  throws IOException
  {
    StringBuffer buf = new StringBuffer();
    int ch;

    InputStreamReader in = (encoding == null)? 
            new FileReader(file) 
            : new InputStreamReader(new FileInputStream(file), encoding);    
    BufferedReader reader = new BufferedReader(in);
    while ((ch = reader.read()) != -1)
    {
      buf.append((char) ch);
    }

    return buf.toString();
  }

  /**
  * Returns a string containing the contents of a file.
  *
  * @param filename the name of the file
  * @return the contents of the file
  */
  public static final String readFile(String filename)  throws IOException
  {
    return readFile(filename, null);
  }
  
  
  /**
   * Returns a string containing the contents of a file.
   *
   * @param filename the name of the file
   * @param encoding  the encoding to use, null means platform default
   * @return the contents of the file 
   */
  public static final String readFile(String filename, String encoding)  throws IOException
  {
    StringBuffer buf = new StringBuffer();
    int ch;

    InputStreamReader in = (encoding == null)? 
            new FileReader(filename) 
            : new InputStreamReader(new FileInputStream(filename), encoding);       
    BufferedReader reader = new BufferedReader(in);
    while ((ch = reader.read()) != -1)
    {
      buf.append((char) ch);
    }

    return buf.toString();
  }
  
  public static final String readReader(Reader r) throws IOException {
	  StringBuffer result = new StringBuffer();
	  
	  BufferedReader reader = new BufferedReader(r);
	  int ch;
	  while ((ch = reader.read()) != -1) {
	      result.append((char) ch);
	  }
	  
	  return result.toString();
  }
  

  public static void main (String [] args)
  {

  /*  File origen = new File("C:\\Proyectos\\pruebasDir\\pp.txt");
    File destino = new File("C:\\Proyectos\\pruebasDir\\paco.txt");
    System.out.println("copiando");
    try
    {
      copy(origen,destino);
      deleteFile(origen);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }*/
    File origen = new File("C:\\Proyectos\\pruebasDir\\paco.txt");
    File destino = new File("C:\\Proyectos\\pruebasDir\\pp.txt");
    origen.renameTo(destino);
    System.out.println("objetos copiados");
  }
  
  /**
   * Method for concatenation of arrays of bytes.
   * @param data first byte array
   * @param tmp second byte array
   * @param size size of second array
   * @return
   */
	public static byte[] concatByteArrays(byte[] data, byte[] tmp) {
		byte[] result = new byte[data.length + tmp.length];
	
		System.arraycopy(data, 0, result, 0, data.length);
		System.arraycopy(tmp, 0, result, data.length, tmp.length);
	
		return result;
	}
}
